/***************************************************************************
 *   Copyright (C) 2008 by Dirk Hartmann                                   *
 *   2monex@gmx.net                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "rddatabasehandler.h"

#include <QSqlDatabase>
#include <QStringList>
#include <QSqlError>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QMap>

QMap<QString, QStringList> RDDatabaseHandler::tableFields;

RDDatabaseHandler::RDDatabaseHandler(QObject * parent)
	: QObject(parent), m_wasRecreated(false)
{
	if (tableFields.isEmpty()) {
		// need to fill in fields 
		// provides table
		tableFields["provides"] << "id" << "requirement" << "providedBy";
		
		// pacakge meta data table
		tableFields["package_meta_data"] << "id" << "packageName" << "architecture" << "packageVersion" << "size" << "fileName" << "shaCheckSum" << "location";
		
		// requires table
		tableFields["requires"] << "id" << "packageName" << "requires";
	}
}

void RDDatabaseHandler::removeDbConnection(const QString & connectionName)
{
	if (QSqlDatabase::contains(connectionName)) {
		QSqlDatabase::removeDatabase(connectionName);
	}
}

void RDDatabaseHandler::setDatabasePath(const QString & database)
{
	databasePath = database;
}

void RDDatabaseHandler::setConnectionName(const QString & name)
{
	connectionName = name;
}

RDDatabaseHandler::~ RDDatabaseHandler()
{
}

bool RDDatabaseHandler::checkDatabase()
{
	// set recreated flag to false
	m_wasRecreated = false;
	
	if (!QSqlDatabase::drivers().contains("QSQLITE")) {
		errMsg = tr("Unable to load database, sqlite database driver not available");
		return false;
	}
	
	QSqlDatabase db;
	if (QSqlDatabase::contains(connectionName)) {
		db = getDatabase();
	} else {
		if (connectionName.isEmpty()) {
			qDebug("empty connection name");
			return false;
		}
	 	db = QSqlDatabase::addDatabase("QSQLITE", connectionName);
		db.setDatabaseName(databasePath);
		
		// open database connection
		if (!db.open()) {
			errMsg = tr("Could not open database: %1").arg(databasePath);
			return false;
		}
	}
	
	QStringList tables = db.tables();
	if (tables.size() == 0) { // create database becuse it's empty
		QSqlError err = initDb();
		if (err.type() == QSqlError::NoError) {
			return true;
		} else {
			errMsg = tr("Could not create database: %1").arg(err.text());
			return false;
		}
	}
	
	if (tables.contains("provides", Qt::CaseInsensitive) 
			&& tables.contains("package_meta_data", Qt::CaseInsensitive)
			&& tables.contains("requires", Qt::CaseInsensitive)) {
		// check database structure
		if (checkDatabaseStructure())
			return true;
		
		// structure wasn't ok recreate database
		qDebug("Recreating Database because of corrupt table structure");
		return recreateDatabase();
	} else {
		qDebug("Recreating Database because of missing tables");
		return recreateDatabase();
	}
	
	return true;
}

bool RDDatabaseHandler::checkDatabaseStructure()
{
	QSqlDatabase db = getDatabase();
	
	foreach (const QString &table, tableFields.keys()) {
		QSqlRecord rec = db.record(table);
		
		foreach(const QString &field, tableFields.value(table)) {
			if (!rec.contains(field))
				return false;
		}
	}
	
	return true;
}

void RDDatabaseHandler::clearContent()
{
	if (connectionName.isEmpty())
		return;
	
	QSqlDatabase db = getDatabase();
	QSqlQuery q(db);
	
	q.exec("delete from package_meta_data");
	q.exec("delete from provides");
	q.exec("delete from requires");
}

QSqlDatabase RDDatabaseHandler::getDatabase()
{
	if (connectionName.isEmpty()) {
		qCritical("Error: No connection name specified");
		return QSqlDatabase();
	}
	
	return QSqlDatabase::database(connectionName);
}

QSqlError RDDatabaseHandler::initDb()
{
	QSqlDatabase db = getDatabase();
	
	QSqlQuery q(db);
	
	if (!q.exec("create table provides(id integer primary key autoincrement, requirement varchar, providedBy varchar)"))
		return q.lastError();
	if (!q.exec("create table package_meta_data(id integer primary key autoincrement, packageName varchar, architecture varchar, packageVersion varchar, size int, fileName varchar, shaCheckSum varchar, location varchar)"))
		return q.lastError();
	if (!q.exec("create table requires(id integer primary key autoincrement, packageName varchar, requires varchar(1024))"))
		return q.lastError();
	
	m_wasRecreated = true;
	
	return QSqlError();
}

bool RDDatabaseHandler::recreateDatabase()
{
	QSqlDatabase db = getDatabase();
	QStringList tables = db.tables();
	QSqlQuery q(db);
	
	foreach(const QString &table, tables) {
		q.exec(QString("drop table %1").arg(table));
	}
	
	QSqlError err = initDb();
	if (err.type() == QSqlError::NoError) {
		m_wasRecreated = true;
		return true;
	}
	
	return false;
}

